@@ -64,6 +64,39 @@ using namespace QtGrpc;
     {serializationFormat}, or other options by constructing it with a
     QGrpcChannelOptions containing the required customizations.
 
+    \section2 Transportation scheme
+
+    The QGrpcHttp2Channel implementation prefers different transportation
+    methods based on the provided \c{hostUri}, \l{QUrl::}{scheme} and options.
+    The following criteria applies:
+
+    \table
+    \header
+        \li Scheme
+        \li Description
+        \li Default Port
+        \li Requirements
+        \li Example
+    \row
+        \li \c{http}
+        \li Unencrypted HTTP/2 over TCP
+        \li 80
+        \li None
+        \li \c{http://localhost}
+    \row
+        \li \c{https}
+        \li TLS-encrypted HTTP/2 over TCP
+        \li 443
+        \li QSslSocket support \b{AND} (scheme \b{OR} \l{QGrpcChannelOptions::}{sslConfiguration})
+        \li \c{https://localhost}
+    \row
+        \li \c{unix}
+        \li Unix domain socket in filesystem path
+        \li ✗
+        \li QLocalSocket support \b{AND} scheme
+        \li \c{unix:///tmp/grpc.socket}
+    \endtable
+
     \section2 Content-Type
 
     The \e{content-type} in \gRPC over HTTP/2 determines the message
@@ -266,7 +299,9 @@ public:
         return false;
 #endif
     }
-    [[nodiscard]] QByteArrayView contentType() const { return m_contentType; }
+    [[nodiscard]] const QByteArray &contentType() const { return m_contentType; }
+
+    [[nodiscard]] const QByteArray &authorityHeader() const { return m_authorityHeader; }
 
     std::shared_ptr<QAbstractProtobufSerializer> serializer;
     QUrl hostUri;
@@ -276,13 +311,22 @@ private:
     enum ConnectionState { Connecting = 0, Connected, Error };
 
     template <typename T>
-    static void connectErrorHandler(T *socket, QGrpcOperationContext *operationContext)
+    void connectErrorHandler(T *socket, QGrpcOperationContext *operationContext)
     {
         QObject::connect(socket, &T::errorOccurred, operationContext,
-                         [operationContextPtr = QPointer(operationContext)](auto error) {
+                         [operationContextPtr = QPointer(operationContext), this](auto error) {
+                             if (m_isInsideSocketErrorOccurred) {
+                                 qGrpcCritical("Socket errorOccurred signal triggered while "
+                                               "already handling an error");
+                                 return;
+                             }
+                             m_isInsideSocketErrorOccurred = true;
+                             auto reset = qScopeGuard([this]() {
+                                 m_isInsideSocketErrorOccurred = false;
+                             });
                              emit operationContextPtr->finished(QGrpcStatus{
                                  StatusCode::Unavailable,
-                                 QGrpcHttp2ChannelPrivate::tr("Network error occurred %1")
+                                 QGrpcHttp2ChannelPrivate::tr("Network error occurred: %1")
                                      .arg(error) });
                          });
     }
@@ -301,6 +345,7 @@ private:
     }
 
     std::unique_ptr<QIODevice> m_socket = nullptr;
+    bool m_isInsideSocketErrorOccurred = false;
     QHttp2Connection *m_connection = nullptr;
     QList<Http2Handler *> m_activeHandlers;
     QList<Http2Handler *> m_pendingHandlers;
@@ -311,6 +356,7 @@ private:
     ConnectionState m_state = Connecting;
     std::function<void()> m_reconnectFunction;
 
+    QByteArray m_authorityHeader;
     Q_DISABLE_COPY_MOVE(QGrpcHttp2ChannelPrivate)
 };
 
@@ -460,12 +506,12 @@ void Http2Handler::prepareInitialRequest(QGrpcOperationContext *operationContext
     QByteArray service{ operationContext->service().data(), operationContext->service().size() };
     QByteArray method{ operationContext->method().data(), operationContext->method().size() };
     m_initialHeaders = HPack::HttpHeader{
-        { AuthorityHeader.toByteArray(),          channel->hostUri.host().toLatin1()       },
+        { AuthorityHeader.toByteArray(),          channel->authorityHeader()               },
         { MethodHeader.toByteArray(),             "POST"_ba                                },
         { PathHeader.toByteArray(),               QByteArray('/' + service + '/' + method) },
         { SchemeHeader.toByteArray(),
          channel->isLocalSocket() ? "http"_ba : channel->hostUri.scheme().toLatin1()       },
-        { ContentTypeHeader.toByteArray(),        channel->contentType().toByteArray()     },
+        { ContentTypeHeader.toByteArray(),        channel->contentType()                   },
         { GrpcServiceNameHeader.toByteArray(),    { service }                              },
         { GrpcAcceptEncodingHeader.toByteArray(), "identity,deflate,gzip"_ba               },
         { AcceptEncodingHeader.toByteArray(),     "identity,gzip"_ba                       },
@@ -656,6 +702,7 @@ QGrpcHttp2ChannelPrivate::QGrpcHttp2ChannelPrivate(const QUrl &uri, QGrpcHttp2Ch
                              QString::fromLatin1(it.value()));
     }
 
+    bool nonDefaultPort = false;
 #if QT_CONFIG(localserver)
     if (hostUri.scheme() == "unix"_L1) {
         auto *localSocket = initSocket<QLocalSocket>();
@@ -681,6 +728,8 @@ QGrpcHttp2ChannelPrivate::QGrpcHttp2ChannelPrivate(const QUrl &uri, QGrpcHttp2Ch
         auto *sslSocket = initSocket<QSslSocket>();
         if (hostUri.port() < 0) {
             hostUri.setPort(443);
+        } else {
+            nonDefaultPort = hostUri.port() != 443;
         }
 
         if (const auto userSslConfig = channelOptions.sslConfiguration(); userSslConfig) {
@@ -719,6 +768,8 @@ QGrpcHttp2ChannelPrivate::QGrpcHttp2ChannelPrivate(const QUrl &uri, QGrpcHttp2Ch
         auto *httpSocket = initSocket<QTcpSocket>();
         if (hostUri.port() < 0) {
             hostUri.setPort(80);
+        } else {
+            nonDefaultPort = hostUri.port() != 80;
         }
 
         QObject::connect(httpSocket, &QAbstractSocket::connected, this,
@@ -735,6 +786,13 @@ QGrpcHttp2ChannelPrivate::QGrpcHttp2ChannelPrivate(const QUrl &uri, QGrpcHttp2Ch
             httpSocket->connectToHost(hostUri.host(), static_cast<quint16>(hostUri.port()));
         };
     }
+
+    m_authorityHeader = hostUri.host().toLatin1();
+    if (nonDefaultPort) {
+        m_authorityHeader += ':';
+        m_authorityHeader += QByteArray::number(hostUri.port());
+    }
+
     m_reconnectFunction();
 }
 
@@ -746,7 +804,7 @@ void QGrpcHttp2ChannelPrivate::processOperation(const std::shared_ptr<QGrpcOpera
     Q_ASSERT_X(operationContextPtr != nullptr, "QGrpcHttp2ChannelPrivate::processOperation",
                "operation context is nullptr.");
 
-    if (!m_socket->isWritable()) {
+    if (!m_socket->isWritable() && m_state == ConnectionState::Connected) {
         operationContextAsyncError(operationContextPtr,
                                    QGrpcStatus{ StatusCode::Unavailable,
                                                 m_socket->errorString() });
@@ -775,7 +833,12 @@ void QGrpcHttp2ChannelPrivate::processOperation(const std::shared_ptr<QGrpcOpera
     if (m_state == ConnectionState::Error) {
         Q_ASSERT_X(m_reconnectFunction, "QGrpcHttp2ChannelPrivate::processOperation",
                    "Socket reconnection function is not defined.");
-        m_reconnectFunction();
+        if (m_isInsideSocketErrorOccurred) {
+            qGrpcWarning("Inside socket error handler. Reconnect deferred to event loop.");
+            QTimer::singleShot(0, [this]{ m_reconnectFunction(); });
+        } else {
+            m_reconnectFunction();
+        }
         m_state = ConnectionState::Connecting;
     }
 }
@@ -785,6 +848,17 @@ void QGrpcHttp2ChannelPrivate::createHttp2Connection()
     Q_ASSERT_X(m_connection == nullptr, "QGrpcHttp2ChannelPrivate::createHttp2Connection",
                "Attempt to create the HTTP/2 connection, but it already exists. This situation is "
                "exceptional.");
+
+    // Nagle's algorithm slows down gRPC communication when frequently sending small utility
+    // HTTP/2 frames. Since an ACK is not sent until a predefined timeout if the TCP frame is
+    // not full enough, communication hangs. In our case, this results in a 40ms delay when
+    // WINDOW_UPDATE or PING frames are sent in a separate TCP frame.
+    //
+    // TODO: We should probably allow users to opt out of this using QGrpcChannelOptions,
+    // see QTBUG-134428.
+    if (QAbstractSocket *abstractSocket = qobject_cast<QAbstractSocket *>(m_socket.get()))
+        abstractSocket->setSocketOption(QAbstractSocket::LowDelayOption, 1);
+
     m_connection = QHttp2Connection::createDirectConnection(m_socket.get(), {});
 
     if (m_connection) {
@@ -854,7 +928,8 @@ void QGrpcHttp2ChannelPrivate::deleteHandler(Http2Handler *handler)
 ///
 
 /*!
-    Constructs QGrpcHttp2Channel with \a hostUri.
+    Constructs QGrpcHttp2Channel with \a hostUri. Please see the
+    \l{Transportation scheme} section for more information.
 */
 QGrpcHttp2Channel::QGrpcHttp2Channel(const QUrl &hostUri)
     : d_ptr(std::make_unique<QGrpcHttp2ChannelPrivate>(hostUri, this))
@@ -862,7 +937,8 @@ QGrpcHttp2Channel::QGrpcHttp2Channel(const QUrl &hostUri)
 }
 
 /*!
-    Constructs QGrpcHttp2Channel with \a hostUri and \a options.
+    Constructs QGrpcHttp2Channel with \a hostUri and \a options. Please see the
+    \l{Transportation scheme} section for more information.
 */
 QGrpcHttp2Channel::QGrpcHttp2Channel(const QUrl &hostUri, const QGrpcChannelOptions &options)
     : QAbstractGrpcChannel(options),
